﻿using System.Diagnostics.CodeAnalysis;

namespace anbs_cp.Extensions;

/// <summary>
/// Расширение Guid
/// </summary>
public static class GuidExtensions
{
    /// <summary>
    /// Проверяет Guid на пустоту
    /// </summary>
    /// <param name="g">Guid или null</param>
    /// <returns>Guid пуст (null) или равен Guid.Empty</returns>
    public static bool IsNullOrEmpty ([NotNullWhen(false)] this Guid? g) => g == null || g == Guid.Empty;

    /// <summary>
    /// Проверяет Guid на пустоту
    /// </summary>
    /// <param name="g">Guid</param>
    /// <returns>Guid пуст (null) или равен Guid.Empty</returns>
    public static bool IsNullOrEmpty (this Guid g) => g == Guid.Empty;

    /// <summary>
    /// Генерирует <see cref="Guid"/>, удовлетворяющий условию <paramref name="predicate"/>
    /// </summary>
    /// <param name="predicate">Условие генерации <see cref="Guid"/></param>
    /// <returns>Новый <see cref="Guid"/></returns>
    public static Guid GenerateGuid (Func<Guid, bool> predicate)
    {
        // Задаю GUID
        Guid guid;

        // Выполняй
        do
        {
            // - генерируй GUID
            guid = Guid.NewGuid();
        }
        // - пока не выполнено условие
        while (!predicate(guid));

        // Возвращаю сгенерированный GUID
        return guid;
    }

    /// <summary>
    /// Генерирует <see cref="Guid"/>, удовлетворяющий условию <paramref name="asyncPredicate"/>
    /// </summary>
    /// <param name="asyncPredicate">Условие генерации <see cref="Guid"/></param>
    /// <returns>Новый <see cref="Guid"/></returns>
    public static async Task<Guid> GenerateGuidAsync (Func<Guid, Task<bool>> asyncPredicate)
    {
        // Задаю GUID
        Guid guid;

        // Выполняй
        do
        {
            // - генерируй GUID
            guid = Guid.NewGuid();
        }
        // - пока не выполнено условие
        while (!await asyncPredicate(guid));

        // Возвращаю сгенерированный GUID
        return guid;
    }
}